package SwissMap;
import drawing.ImageViewer; 

// Vertex class only used locally for polygons
class Vertex {
	public int x, y;
	public Vertex next;
	
	Vertex(int x0, int y0, Vertex nxt)
	{
		next = nxt;
		x = x0;
		y = y0;
	}
}

public class Polygon {
	
	Vertex first, last; // linked circular list of vertices
	String name;
	
	Polygon(String n)
	{
		first = null;
		last = null;
		name = n;
	}
	
	// add point to the linked circular list
	// pre: first = null or v.next != null for all vertices v
	public void AddPoint(int x, int y)
	{
		Vertex v = new Vertex(x,y,first);
		if (first == null){
			first = v;
			last = v;
		}
		last.next = v;
		last = v;
	}
	
	public void DrawName (ImageViewer p)
	{
		if (first == last) return;
		Vertex v = first;
		int minx = v.x;
		int maxx = v.x;
		int miny = v.y;
		int maxy = v.y;
		miny = v.y;
		v = v.next;
		while (v != first){
			if (v.x < minx) minx = v.x;
			if (v.x > maxx) maxx = v.x;
			if (v.y < miny) miny = v.y;
			if (v.y > maxy) maxy = v.y;
			v = v.next;
		}
		p.PutText((maxx+minx)/2,(maxy+miny)/2,name);
	}
	
	public void DrawPolygon(ImageViewer p)
	{
		if (first == last) // includes both cases first == last == null and first == last != null
			return;
		Vertex v = first;
		do{
			p.line(v.x,v.y,v.next.x,v.next.y);
			v = v.next;
		}
		while (v != first);

	}
	
	public boolean IntersectHorizontalLeft(int x, int y, Vertex a, Vertex b)
	{
		if (a.y < y && y <= b.y || b.y < y && y <= a.y) // schnitt
		{
			double sx = a.x + (b.x-a.x) * (double) (y-a.y) / (b.y-a.y);
			return sx <= x; // Schnittpunkt links
		}
		else
			return false;
	}
	
	public boolean PointInPolygon(int px, int py)
	{
		if (first == last) // degeneriert 
			return false;
		else // first != last
		{
			int count = 0;
			Vertex v = first;
			do{ // Kante von v nach v.next;
				// z�hle Kanten mit echtem Schnitt
				if (IntersectHorizontalLeft(px, py, v, v.next))
				{	count ++;
				}
				v = v.next;
			} while (v != first);
			return count % 2 == 1;			
		}
	}
	

	
}
