package drawing;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import javax.swing.JFrame;
import javax.swing.JPanel;

public class ImageViewer extends JPanel{
	
	static final long serialVersionUID=0;

    private BufferedImage canvas;
    private Color color;

    // required in order to work 
    public Dimension getPreferredSize() {
        return new Dimension(canvas.getWidth(), canvas.getHeight());
    }
    // required in order to work
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;
        g2.drawImage(canvas, null, null);
    }
    
    // open existing image
    private ImageViewer(BufferedImage img) {
	    JFrame frame = new JFrame("ImageView");
        canvas = img ;
		frame.add(this);
	    frame.pack();
	    frame.setVisible(true);
	    frame.setResizable(false);
	    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	    color = Color.black;
    }

    //-------------------------------------------------------------------------------
    // students: relevant for you is only the following part of the public interface 
    // ------------------------------------------------------------------------------
    
    // open empty image with dimensions width x height
    public ImageViewer(int width, int height)
    {
		this(new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB));
    }

    // empty image
    public void clear()
    {
        for (int y=0; y<canvas.getHeight(); ++y)
        	for (int x=0; x<canvas.getWidth(); ++x)
        		canvas.setRGB(x, y, 0);
        repaint();
    }

    public void clear(Color col)
    {
        for (int y=0; y<canvas.getHeight(); ++y)
        	for (int x=0; x<canvas.getWidth(); ++x)
        		canvas.setRGB(x, y, col.getRGB());
        repaint();
    }

    // draw pixel at x,y
    // some safety: if out of bounds of the image, do nothing
    public void dot(int x, int y)
    {
    	
    	if (x<0 || y <0) return;
    	if (x>=canvas.getWidth() || y >= canvas.getHeight()) return;
    	
    	canvas.setRGB(x, y, color.getRGB());
    }
    
    // compact Bresenham: draw line from x0,y0 to x1,y1, source Wikipedia
	public void line(int x0, int y0, int x1, int y1)
	{
		int dx =  Math.abs(x1-x0), sx = x0<x1 ? 1 : -1;
		int dy = -Math.abs(y1-y0), sy = y0<y1 ? 1 : -1;
		int err = dx+dy, e2; /* error value e_xy */

		for(;;){  /* loop */
			dot(x0, y0);;
			if (x0==x1 && y0==y1) break;
			e2 = 2*err;
			if (e2 > dy) { err += dy; x0 += sx; } /* e_xy+e_x > 0 */
			if (e2 < dx) { err += dx; y0 += sy; } /* e_xy+e_y < 0 */
		}
	}
	
	// compact Bresenham ellipse,source: Wikipedia
	public void ellipse(int xm, int ym, int a, int b)
	{
	   int dx = 0, dy = b; /* im I. Quadranten von links oben nach rechts unten */
	   long a2 = a*a, b2 = b*b;
	   long err = b2-(2*b-1)*a2, e2; /* Fehler im 1. Schritt */
	 
	   do {
	       dot(xm+dx, ym+dy); /* I. Quadrant */
	       dot(xm-dx, ym+dy); /* II. Quadrant */
	       dot(xm-dx, ym-dy); /* III. Quadrant */
	       dot(xm+dx, ym-dy); /* IV. Quadrant */
	 
	       e2 = 2*err;
	       if (e2 <  (2*dx+1)*b2) { dx++; err += (2*dx+1)*b2; }
	       if (e2 > -(2*dy-1)*a2) { dy--; err -= (2*dy-1)*a2; }
	   } while (dy >= 0);
	 
	   while (dx++ < a) { /* fehlerhafter Abbruch bei flachen Ellipsen (b=1) */
	       dot(xm+dx, ym); /* -> Spitze der Ellipse vollenden */
	       dot(xm-dx, ym);
	   }
	}
	
	public void PutText(int x,int y, String text)
	{
       Graphics2D g2d = canvas.createGraphics();
       g2d.setPaint(Color.red);
       g2d.setFont(new Font("Serif", Font.CENTER_BASELINE+Font.PLAIN, 10));
       FontMetrics fm = g2d.getFontMetrics();
       x = x - fm.stringWidth(text) / 2;
       g2d.drawString(text, x, y);
       g2d.dispose();
	}

	public void setColor(Color c)
	{
		color = c;
	}
    // update content
    public void update()
    {
    	repaint();
    }

}

